/********************************************************************
    Project :   PE5004 Evaboard
    Version :   v1.1             
    Software:   PSoC Designer 
    Hardware:   PSoC MiniProg, PE5004 Evaboard
	Device	: 	CY7C64356
    Date    :   Jan 29 2009
    Author  :   HoJ
    Company :   PE Gmbh   

	desgription: Library S64128K Series LCD Module (128 x 64 DOTS)
				 source code
				 
    LCD_Data_Port = Port0
    
    /WR  = PORT2.0
    /RD  = PORT2.1
    A0   = PORT2.2
    /CS1 = PORT2.3
    /RES = PORT2.4
    
******************************************************************************/

void DISPLAY_HARD_RESET(void)
{
    LCD_CONT_PORT = 0b00001111; // A0  Low --> COMMAND, rest High
    LCD_CONT_PORT = 0b00011111; // /WR and /CS1 Low
}

void WRITE_COMMAND(BYTE DISPLAY_COMMAND)
{  
    LCD_DATA_PORT_DRIVE_0  = 0b11111111; //Drive Mode strong/strong
    LCD_DATA_PORT_DRIVE_1  = 0b00000000;
    LCD_CONT_PORT = 0b00011011; // A0  Low --> COMMAND, rest High
    LCD_CONT_PORT = 0b00010010; // /WR and /CS1 Low
    LCD_DATA_PORT = DISPLAY_COMMAND; // write COMMAND
    LCD_CONT_PORT = 0b00011011; // /WR and /CS1 High
    LCD_CONT_PORT = 0b00011111; // A0  High
}

void WRITE_COMMAND_FAST(BYTE DISPLAY_COMMAND)
{
    LCD_CONT_PORT = 0b00011011; // A0  Low --> COMMAND, rest High
    LCD_CONT_PORT = 0b00010010; // /WR and /CS1 Low
    LCD_DATA_PORT = DISPLAY_COMMAND; // write COMMAND
    LCD_CONT_PORT = 0b00011111; // A0  High
}

void WRITE_DATA(BYTE DISPLAY_DATA)
{
    LCD_DATA_PORT_DRIVE_0  = 0b11111111; //Drive Mode strong/strong
    LCD_DATA_PORT_DRIVE_1  = 0b00000000;
    LCD_CONT_PORT = 0b00011111; // A0  High --> DATA, rest also High
    LCD_CONT_PORT = 0b00010110; // /WR and /CS1 Low    
    LCD_DATA_PORT = DISPLAY_DATA; // write Data
    LCD_CONT_PORT = 0b00011111; // /WR and /CS1 High
}

void WRITE_DATA_FAST(BYTE DISPLAY_DATA)
{
    LCD_CONT_PORT = 0b00011111; // A0  High --> DATA, rest also High
    LCD_CONT_PORT = 0b00010110; // /WR and /CS1 Low   
    LCD_DATA_PORT = DISPLAY_DATA; // write Data
    LCD_CONT_PORT = 0b00011111; // /WR and /CS1 High
}

BYTE READ_DATA(void)
{
	BYTE DISPLAY_DATA_RECEIVED;
    
	LCD_DATA_PORT = 0b11111111; 
	
    LCD_DATA_PORT_DRIVE_0  = 0b11111111; //Drive Mode High Z/ High Z
    LCD_DATA_PORT_DRIVE_1  = 0b11111111;
    
    LCD_CONT_PORT = 0b00011111; // A0  High --> DATA, rest also High
    LCD_CONT_PORT = 0b00010101; // /RD and /CS1 Low dummy read
    LCD_CONT_PORT = 0b00010111; // /RD High and /CS1 High
    LCD_CONT_PORT = 0b00010101; // /RD and /CS1 Low 
    DISPLAY_DATA_RECEIVED = LCD_DATA_PORT; // read Data
    LCD_CONT_PORT = 0b00011111; // /RD and /CS1 High
    return DISPLAY_DATA_RECEIVED;
}

void DISPLAY_ON(void)
{
    WRITE_COMMAND(DISPLAY_ON_C);
}

void DISPLAY_OFF(void)
{
    WRITE_COMMAND(DISPLAY_OFF_C);
}

void SET_DISPLAY_START_LINE(BYTE START_LINE_SUB)
{
    WRITE_COMMAND((0b01000000) | (START_LINE_SUB)); // (0b01000000 + START_LINE_SUB)
}

void SET_DISPLAY_PAGE_ADRESS(BYTE PAGE_ADRESS_SUB)
{
    WRITE_COMMAND((0b10110000) | (PAGE_ADRESS_SUB));
}

void SET_DISPLAY_COLUMN_ADRESS(BYTE COLUMN_ADRESS_SUB)
{
    WRITE_COMMAND((0b00010000) | (COLUMN_ADRESS_SUB >> 4));
    WRITE_COMMAND((0b00001111) & (COLUMN_ADRESS_SUB));
}

void SET_DISPLAY_COLUMN_ADRESS_FAST(BYTE COLUMN_ADRESS_SUB)
{
    WRITE_COMMAND_FAST((0b00010000) | (COLUMN_ADRESS_SUB >> 4));
    WRITE_COMMAND_FAST((0b00001111) & (COLUMN_ADRESS_SUB));
}

void ADC_NORMAL(void)
{
    WRITE_COMMAND(ADC_SELECT_NORMAL_C);
}

void ADC_REVERSE(void)
{
    WRITE_COMMAND(ADC_SELECT_REVERSE_C);
}

void DISPLAY_NORMAL(void)
{
    WRITE_COMMAND(DISPLAY_NORMAL_C);
}

void DISPLAY_REVERSE(void)
{
    WRITE_COMMAND(DISPLAY_REVERSE_C);
}

void DISPLAY_ALL_POINTS_ON(void)
{
    WRITE_COMMAND(DISPLAY_ALL_POINTS_ON_C);
}

void DISPLAY_ALL_POINTS_OFF(void)
{
    WRITE_COMMAND(DISPLAY_ALL_POINTS_OFF_C);
}

void LCD_BIAS_SET_1_9(void)
{
    WRITE_COMMAND(LCD_BIAS_SET_1_9_C);
}

void LCD_BIAS_SET_1_7(void)
{
    WRITE_COMMAND(LCD_BIAS_SET_1_7_C);
}

void LCD_INTERNAL_RESET(void)
{
    WRITE_COMMAND(DISPLAY_RESET_C);
}

void COMMON_OUTPUT_MODE_NORMAL(void)
{
    WRITE_COMMAND(0b11000000);
}

void COMMON_OUTPUT_MODE_REVERSE(void)
{
    WRITE_COMMAND(0b11001000);
}

void POWER_CONTROL_SET(BYTE POWER_MODE_SUB)
{
    WRITE_COMMAND((0b00101000) | (POWER_MODE_SUB));
}

void VOLTAGE_REGU_RES_RATIO_SET(BYTE VOLTAGE_REGU_RES_RATIO_SUB)
{
    WRITE_COMMAND((0b00100000) | (VOLTAGE_REGU_RES_RATIO_SUB));
}

void ELEC_VOL_MODE_SET(BYTE ELEC_VOL_MODE_REG_SUB)
{
    WRITE_COMMAND(ELEC_VOL_MODE_SET_C);
    WRITE_COMMAND(ELEC_VOL_MODE_REG_SUB);
}

void STATIC_INDICAT_ON_MODE(BYTE STATIC_INDICAT_MODE_SUB)
{
    WRITE_COMMAND(STATIC_INDICAT_ON_C);
    WRITE_COMMAND(STATIC_INDICAT_MODE_SUB);
}

void STATIC_INDICAT_OFF(void)
{
    WRITE_COMMAND(STATIC_INDICAT_OFF_C);
    WRITE_COMMAND(0b00000000);
}

void BOOSTER_RATIO_SET(BYTE STEP_UP_VALUE_SUB)
{
    WRITE_COMMAND(0b11111000);
    WRITE_COMMAND(STEP_UP_VALUE_SUB);
}

void READ_MODIFY_WRITE(void)
{
    WRITE_COMMAND(0b11100000);
}

void READ_MODIFY_WRITE_END(void)
{
    WRITE_COMMAND(0b11101110);
}

void NOP_COMMAND(void)
{
    WRITE_COMMAND(NOP_COMMAND_C);
}

void DISPLAY_INIT_SINGLE_SUPPLY(void)
{
    DISPLAY_HARD_RESET(); // Reset Pin ?
    SET_DISPLAY_START_LINE(0x00);  //Display start line 0
    //ADC_NORMAL();                //ADC normal
    ADC_REVERSE();                 //ADC reverse
    COMMON_OUTPUT_MODE_NORMAL();  //Normal COM0-COM63
    //COMMON_OUTPUT_MODE_REVERSE();  //Reverse COM63-COM0
    DISPLAY_NORMAL();              //Display normal
    LCD_BIAS_SET_1_9();            //Set bias 1/9 (Duty 1/65)
    POWER_CONTROL_SET(0b00000111); //Booster, Regulator and Follower on
    BOOSTER_RATIO_SET(0x00);       //Set internal Booster to 4x
    VOLTAGE_REGU_RES_RATIO_SET(0b00000101);  // Contrast set
    ELEC_VOL_MODE_SET(0b00010110);           //     "
    STATIC_INDICAT_OFF();          //No indicator 
    DISPLAY_ON();                  //Display on
}

void DISPLAY_SET_PIXEL(BYTE x, BYTE y, BYTE on_off)
{
	BYTE OLD_DISPLAY_RAM_VAR;
	BYTE DISPLAY_PAGE;
	if (x < DISPLAY_WIDTH && y < DISPLAY_HEIGHT){
        DISPLAY_PAGE = (y >> 3);
        //read previous display ram value
        y &= 0b00000111; 
        READ_MODIFY_WRITE(); //does not change the column address
        SET_DISPLAY_PAGE_ADRESS(DISPLAY_PAGE);
        SET_DISPLAY_COLUMN_ADRESS(x);
        OLD_DISPLAY_RAM_VAR = READ_DATA();
        if (on_off != 0){    
            OLD_DISPLAY_RAM_VAR |= (1 << y); //set new bit, modify old value!
        	}
        else{
            OLD_DISPLAY_RAM_VAR &= ~(1 << y); //clear new bit, modify old value!
        	}
        WRITE_DATA(OLD_DISPLAY_RAM_VAR);
        READ_MODIFY_WRITE_END();
    	}
}

void DISPLAY_CLEAR(void)
{
	//clears the display RAM
    BYTE page;
	BYTE column;
	for (page = 0; page < 8; page++){
	    WRITE_COMMAND(0xB0 + page); //Set page address to <page>
	    WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
	    WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	    for (column = 0; column < 0x84; column++)
	    WRITE_DATA(0x00);
		}
}

void GRAPH_DRAW_DATA_COLUMN_CLEAR(BYTE x, BYTE y, BYTE data)
{
	BYTE row;
	for (row = 0; row < 8; row++,y++){
		if ((data & (1<<row)) != 0)
			DISPLAY_SET_PIXEL(x, y, 1);
		else
			DISPLAY_SET_PIXEL(x, y, 0);
		}
}

BYTE GRAPH_DRAW_CHAR_CLEAR(BYTE x, BYTE y, BYTE c)
{
	BYTE w = 0; //width of drawn character (return value)
	BYTE addr = 0; 
	BYTE data = 0x00;
    GRAPH_DRAW_DATA_COLUMN_CLEAR(x-1, y, 0x00);
	while (data != 0xAA && w++ < 7){
		data = FONT_STANDARD[c-0x20][addr];
		if (data != 0xAA)
			GRAPH_DRAW_DATA_COLUMN_CLEAR(x++, y, data);
		else
			w--;
		addr++;
		}
	if (w > 7) w = 7;
	return (w);
}

void GRAPH_DRAW_LOGO(BYTE x, BYTE y, BYTE r, BYTE c)
{
	WORD addr = 0; 
	BYTE column = x;
	BYTE row = y;
	BYTE data = 0x00;
	while (row < (r+y)){
		while (column < (c+x)){	
			data = Logo_sw_small2bmp[addr];
			GRAPH_DRAW_DATA_COLUMN_CLEAR(column++, row, data);
			addr++;
			}
		column = x;
		row = row + 8;
		}
}

void GRAPH_DRAW_STRING_ref(BYTE x, BYTE y,const char *s)
{
		while (*s){   // so long *s != '\0' "string-end"
			x += GRAPH_DRAW_CHAR_CLEAR(x, y, *s) + 1;
			s++;
			}
}

void DISPLAY_SET_PIXEL_RAM(BYTE x, BYTE y, BYTE on_off)
{
	BYTE DISPLAY_PAGE;
	if (x < DISPLAY_WIDTH && y < DISPLAY_HEIGHT){
        DISPLAY_PAGE = y >> 3;
        y &= 0b00000111; 
		if (DISPLAY_PAGE == 0){
        	if (on_off != 0){    
            	disp_ram_page0[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page0[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 1){
        	if (on_off != 0){    
            	disp_ram_page1[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page1[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 2){
        	if (on_off != 0){    
            	disp_ram_page2[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page2[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 3){
        	if (on_off != 0){    
            	disp_ram_page3[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page3[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 4){
        	if (on_off != 0){    
            	disp_ram_page4[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page4[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 5){
        	if (on_off != 0){    
            	disp_ram_page5[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page5[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 6){
        	if (on_off != 0){    
            	disp_ram_page6[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page6[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
		if (DISPLAY_PAGE == 7){
        	if (on_off != 0){    
            	disp_ram_page7[x] |= (1 << y); //set new bit, modify old value!
        		}
        	else{
            	disp_ram_page7[x] &= ~(1 << y); //clear new bit, modify old value!
        		}
			}
    	}
}

void CLEAR_RAM(void) //clear display MC ram
{
		BYTE column;
		for (column = 0; column < 128; column++){
			disp_ram_page0[column]=0x00;
			disp_ram_page1[column]=0x00;
			disp_ram_page2[column]=0x00;
			disp_ram_page3[column]=0x00;
			disp_ram_page4[column]=0x00;
			disp_ram_page5[column]=0x00;
			disp_ram_page6[column]=0x00;
			disp_ram_page7[column]=0x00;
			}
}

void GRAPH_DRAW_DATA_COLUMN_CLEAR_RAM(BYTE x, BYTE y, BYTE data)
{
	BYTE row;
	for (row = 0; row < 8; row++,y++){
		if ((data & (1<<row)) != 0)
			DISPLAY_SET_PIXEL_RAM(x, y, 1);
		else
			DISPLAY_SET_PIXEL_RAM(x, y, 0);
		}
}

void GRAPH_DRAW_DATA_COLUMN_TRANSPARENT_RAM(BYTE x, BYTE y, BYTE data)
{
	BYTE row;
	for (row = 0; row < 8; row++,y++){
		if ((data & (1<<row)) != 0)
			DISPLAY_SET_PIXEL_RAM(x, y, 1);
	//	else
	//		DISPLAY_SET_PIXEL_RAM(x, y, 0);
		}
}

void GRAPH_DRAW_DATA_COLUMN_RAM(BYTE x, BYTE y, BYTE data)
{
	BYTE row;
	for (row = 0; row < 8; row++,y++){
		if ((data & (1<<row)) != 0)
			DISPLAY_SET_PIXEL_RAM(x, y, 1);
		else
			DISPLAY_SET_PIXEL_RAM(x, y, 0);
		}
}

BYTE GRAPH_DRAW_CHAR_RAM(BYTE x, BYTE y, BYTE c)
{
	BYTE w = 0; //width of drawn character (return value)
	BYTE addr = 0; 
	BYTE data = 0x00;
    GRAPH_DRAW_DATA_COLUMN_RAM(x-1, y, 0x00);
	while (data != 0xAA && w++ < 7){
		data = FONT_STANDARD[c-0x20][addr];
		if (data != 0xAA)
			GRAPH_DRAW_DATA_COLUMN_RAM(x++, y, data);
		else
			w--;
		addr++;
		}
	if (w > 7) w = 7;
	return (w);
}

void GRAPH_DRAW_STRING_RAM(BYTE x, BYTE y,const char *s)
{
		while (*s){   
		    x += GRAPH_DRAW_CHAR_RAM(x, y, *s) + 1;
			s++;
			}
}

void DISPLAY_GRAPH_CURSOR(BYTE x, BYTE y)
{
	BYTE addr = 0; 
	BYTE data = 0x00;
	while (addr < 5){
		data = CURSOR_FONT[addr];
		GRAPH_DRAW_DATA_COLUMN_TRANSPARENT_RAM(x++, y, data);
		addr++;
		}
}

void DISP_SEND_FRAME(void)
{
		BYTE column;
		WRITE_COMMAND(0xB0); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page0[column]);

		WRITE_COMMAND(0xB0 + 1); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page1[column]);

		WRITE_COMMAND(0xB0 + 2); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page2[column]);

		WRITE_COMMAND(0xB0 + 3); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page3[column]);

		WRITE_COMMAND(0xB0 + 4); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page4[column]);

		WRITE_COMMAND(0xB0 + 5); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page5[column]);

		WRITE_COMMAND(0xB0 + 6); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page6[column]);

		WRITE_COMMAND(0xB0 + 7); //Set page address to <page>
		WRITE_COMMAND(0x10 + 0); //Set column address to 0 (4 MSBs)
		WRITE_COMMAND(0x00 + 0); //Set column address to 0 (4 LSBs)
	
		for (column = 0; column < 128; column++)
			WRITE_DATA_FAST(disp_ram_page7[column]);
}
	
void DRAW_GRID_RAM(void) 
{
		BYTE column;
		BYTE array;
		array=0;
		for (column = 40; column < 104; column++){
			disp_ram_page1[column]=GRID_STANDARD[array];
			disp_ram_page2[column]=GRID_STANDARD[array];
			disp_ram_page3[column]=GRID_STANDARD[array];
			disp_ram_page4[column]=GRID_STANDARD[array];
			disp_ram_page5[column]=GRID_STANDARD[array];
			disp_ram_page6[column]=GRID_STANDARD[array];
			disp_ram_page7[column]=GRID_STANDARD[array];
			array++;
			}
}

void DRAW_SLIDER_GRID_RAM(void) 
{
		BYTE column;
		BYTE array;
		array=0;
		for (column = 40; column < 120; column++){
			disp_ram_page1[column]=GRID_SLIDER[array];
			disp_ram_page2[column]=GRID_SLIDER[array];
			disp_ram_page3[column]=GRID_SLIDER[array];
			disp_ram_page4[column]=GRID_SLIDER[array];
			disp_ram_page5[column]=GRID_SLIDER[array];
			disp_ram_page6[column]=GRID_SLIDER[array];
			disp_ram_page7[column]=GRID_SLIDER[array];
			array++;
			}
}

void DISPLAY_GRAPH_SLIDER(BYTE x, BYTE y)
{
	BYTE addr = 0; 
	while (addr < 8){
		DISPLAY_SET_PIXEL_RAM(x, y, 1);
		DISPLAY_SET_PIXEL_RAM(x++, y+1, 1);
		addr++;
		}
}

void GRAPH_DRAW_STRING(BYTE x, BYTE y, char *s)
{
		while (*s)
		{   // so lange *s != '\0' also ungleich dem "String-Endezeichen"
			x += GRAPH_DRAW_CHAR_CLEAR(x, y, *s) + 1;
			s++;
		}
}
