/********************************************************************
    Project :   PE5004 Evaboard
    Version :   v1.1             
    Software:   PSoC Designer 
    Hardware:   PSoC MiniProg, PE5004 Evaboard
	Device	: 	CY7C64356
    Date    :   Jan 29 2009
    Author  :   HoJ
    Company :   PE Gmbh   

	desgription: Library S64128K Series LCD Module (128 x 64 DOTS)
	
    LCD_Data_Port = Port0
    
    /WR  = PORT2.0
    /RD  = PORT2.1
    A0   = PORT2.2
    /CS1 = PORT2.3
    /RES = PORT2.4
    
******************************************************************************/

#include <m8c.h>
#include <font.h> //ascii code
#include <Logo_sw_small.h> //PE logo
//Display properties -----------------------------------------------

#define DISPLAY_WIDTH   0x80
#define DISPLAY_HEIGHT  0x40

//ports/pins defines -----------------------------------------------

#define LCD_DATA_PORT  PRT0DR
#define LCD_DATA_PORT_DRIVE_0  PRT0DM0
#define LCD_DATA_PORT_DRIVE_1  PRT0DM1
#define LCD_CONT_PORT  PRT2DR
#define WR_LCD_PIN     0
#define RD_LCD_PIN     1
#define A0_LCD_PIN     2
#define CS1_LCD_PIN    3
#define RES_LCD_PIN    4

//command defines ---------------------------------------------------
#define DISPLAY_ON_C             0b10101111
#define DISPLAY_OFF_C            0b10101110
#define ADC_SELECT_NORMAL_C      0b10100000
#define ADC_SELECT_REVERSE_C     0b10100001
#define DISPLAY_NORMAL_C         0b10100110
#define DISPLAY_REVERSE_C        0b10100111
#define DISPLAY_ALL_POINTS_ON_C  0b10101000
#define DISPLAY_ALL_POINTS_OFF_C 0b10101001
#define LCD_BIAS_SET_1_9_C       0b10100010
#define LCD_BIAS_SET_1_7_C       0b10100011
#define DISPLAY_RESET_C          0b11100010
#define ELEC_VOL_MODE_SET_C      0b10000001
#define STATIC_INDICAT_ON_C      0b10101100
#define STATIC_INDICAT_OFF_C     0b10101101
#define NOP_COMMAND_C            0b11100011

//MC display ram
BYTE disp_ram_page0[128];
BYTE disp_ram_page1[128];
BYTE disp_ram_page2[128];
BYTE disp_ram_page3[128];
BYTE disp_ram_page4[128];
BYTE disp_ram_page5[128];
BYTE disp_ram_page6[128];
BYTE disp_ram_page7[128];


//-------------------------------------------------
// Prototypes
//-------------------------------------------------

void DISPLAY_HARD_RESET(void);

void WRITE_COMMAND(BYTE DISPLAY_COMMAND);

void WRITE_COMMAND_FAST(BYTE DISPLAY_COMMAND);

void WRITE_DATA(BYTE DISPLAY_DATA);

void WRITE_DATA_FAST(BYTE DISPLAY_DATA);

BYTE READ_DATA(void);

void DISPLAY_ON(void);

void DISPLAY_OFF(void);

void SET_DISPLAY_START_LINE(BYTE START_LINE_SUB);

void SET_DISPLAY_PAGE_ADRESS(BYTE PAGE_ADRESS_SUB);

void SET_DISPLAY_COLUMN_ADRESS(BYTE COLUMN_ADRESS_SUB);

void SET_DISPLAY_COLUMN_ADRESS_FAST(BYTE COLUMN_ADRESS_SUB);

void ADC_NORMAL(void);

void ADC_REVERSE(void);

void DISPLAY_NORMAL(void);

void DISPLAY_REVERSE(void);

void DISPLAY_ALL_POINTS_ON(void);

void DISPLAY_ALL_POINTS_OFF(void);

void LCD_BIAS_SET_1_9(void);

void LCD_BIAS_SET_1_7(void);

void LCD_INTERNAL_RESET(void);

void COMMON_OUTPUT_MODE_NORMAL(void);

void COMMON_OUTPUT_MODE_REVERSE(void);

void POWER_CONTROL_SET(BYTE POWER_MODE_SUB);

void VOLTAGE_REGU_RES_RATIO_SET(BYTE VOLTAGE_REGU_RES_RATIO_SUB);

void ELEC_VOL_MODE_SET(BYTE ELEC_VOL_MODE_REG_SUB);

void STATIC_INDICAT_ON_MODE(BYTE STATIC_INDICAT_MODE_SUB);

void STATIC_INDICAT_OFF(void);

void BOOSTER_RATIO_SET(BYTE STEP_UP_VALUE_SUB);

void READ_MODIFY_WRITE(void);

void READ_MODIFY_WRITE_END(void);

void NOP_COMMAND(void);

void DISPLAY_INIT_SINGLE_SUPPLY(void);

void DISPLAY_SET_PIXEL(BYTE x, BYTE y, BYTE on_off);

void DISPLAY_CLEAR(void);

void GRAPH_DRAW_DATA_COLUMN_CLEAR(BYTE x, BYTE y, BYTE data);

BYTE GRAPH_DRAW_CHAR_CLEAR(BYTE x, BYTE y, BYTE c);

void GRAPH_DRAW_LOGO(BYTE x, BYTE y, BYTE r, BYTE c);

void GRAPH_DRAW_STRING_ref(BYTE x, BYTE y,const char *s);

void DISPLAY_SET_PIXEL_RAM(BYTE x, BYTE y, BYTE on_off);

void CLEAR_RAM(void); //clear display MC ram

void GRAPH_DRAW_DATA_COLUMN_CLEAR_RAM(BYTE x, BYTE y, BYTE data);

void GRAPH_DRAW_DATA_COLUMN_RAM(BYTE x, BYTE y, BYTE data);

BYTE GRAPH_DRAW_CHAR_RAM(BYTE x, BYTE y, BYTE c);

void GRAPH_DRAW_STRING_RAM(BYTE x, BYTE y,const char *s);

void DISPLAY_GRAPH_CURSOR(BYTE x, BYTE y);

void DISP_SEND_FRAME(void);

void DRAW_GRID_RAM(void);

void DRAW_SLIDER_GRID_RAM(void);

void DISPLAY_GRAPH_SLIDER(BYTE x, BYTE y);

//-------------------------------------------------
// source code
//-------------------------------------------------

#include "display.c"
