/********************************************************************
    Project :   PE5004 Evaboard
    Version :   v2.0             
    Software:   PSoC Designer 
    Hardware:   PSoC MiniProg, PE5004 Evaboard
	Device	: 	CY7C64356 
    Date    :   Jan 29 2009
    Author  :   HoJ
    Company :   PE Gmbh               

	desgription: SPI functions source code
	
	PORT_1.0	O	strong	SEL0        (BOARD SENSORS)
	PORT_1.1	O	strong	MOSI        (BOARD SENSORS, EXTERNAL SENSORS)
	PORT_1.2	O	strong	SEL1        (EXTERNAL SENSORS)
	PORT_1.3	O	strong	SPI_CLK  (BOARD SENSORS, EXTERNAL SENSORS)
				
	PORT_3.5	I	open drain bypass	MISO        (BOARD SENSORS, EXTERNAL SENSORS)
	PORT_3.0	I	open drain	INT0  (read modify write!)  (BOARD SENSORS)
	PORT_3.1	I	open drain	INT1 (read modify write!)   (EXTERNAL SENSORS)

******************************************************************************/

//Load default Matrix 5004
void LOAD_Matrix(void){
    
    WaitCOL = WaitCOL_M;
    WaitROW = WaitROW_M;
    SINF = SINF_M;
    SINA = SINA_M;
    PWR =  PWR_M;
    GCO_DACREF = GCO_DACREF_M;
    COLEND_ROWEND = COLEND_ROWEND_M;
	COLSTART_ROWSTART = COLSTART_ROWSTART_M;
    PCO = PCO_M;
}

void LOAD_Matrix_EXIT(void){
    
    WaitCOL = WaitCOL_M;
    WaitROW = WaitROW_M;
    SINF = SINF_M;
    SINA = SINA_M;
    PWR =  PWR_M;
    GCO_DACREF = GCO_DACREF_M;
    COLEND_ROWEND = 0x14;
	COLSTART_ROWSTART = COLSTART_ROWSTART_M;
    PCO = PCO_M;
}

//void LOAD_Matrix_USB(char *inData){
void LOAD_Matrix_USB(void){

    WaitCOL = inData[1];
    WaitROW = inData[2];
    SINF = inData[4];
    SINA = inData[5];
    PWR =  inData[3];
    GCO_DACREF = inData[6];
    PCO = ((inData[8]<<8)|inData[7]); //built 10bit pco value
	COLEND_ROWEND = inData[9];
	COLSTART_ROWSTART = inData[12]; 
}

//Load default OLED 5004
void LOAD_OLED(void){
    
    WaitCOL = WaitCOL_O ;
    WaitROW =  WaitROW_O;
    SINF = SINF_O;
    SINA = SINA_O;
    PWR =  PWR_O;
    GCO_DACREF = GCO_DACREF_O;
    COLEND_ROWEND = COLEND_ROWEND_O;
	COLSTART_ROWSTART = COLSTART_ROWSTART_O;
    PCO = PCO_O; 
}
//Load default OLED EXIT 5004
void LOAD_OLED_EXIT(void){
    
    WaitCOL = WaitCOL_E ;
    WaitROW =  WaitROW_E;
    SINF = SINF_E;
    SINA = SINA_E;
    PWR =  PWR_E;
    GCO_DACREF = GCO_DACREF_E;
    COLEND_ROWEND = COLEND_ROWEND_E;
	COLSTART_ROWSTART = COLSTART_ROWSTART_E;
    PCO = PCO_E;
}


// Sending controll byte + 3 data bytes to 5004 with enabling/disabling SPI
void SPI_SEND_5004(BYTE adress, BYTE send2 ,BYTE send1 ,BYTE send0){

    SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //pull SPI_SEL low
    SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //pull SPI_SEL low //extra delay
    SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //pull SPI_SEL low //extra delay

    SPIM_SendTxData(adress);  //send adress 
    while( ! (SPIM_CONTROL_REG & SPIM_SPIM_TX_BUFFER_EMPTY ) );
    SPIM_SendTxData(send2);  //sending bit 23-16
    while( ! (SPIM_CONTROL_REG & SPIM_SPIM_TX_BUFFER_EMPTY ) );
    SPIM_SendTxData(send1);  //sending bit 15-8
    while( ! (SPIM_CONTROL_REG & SPIM_SPIM_TX_BUFFER_EMPTY ) );
    SPIM_SendTxData(send0);  //sending bit 7-0
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_TX_BUFFER_EMPTY ) );
    while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE) );
	
    SPI_SEL_PORT = 0x25;  // pull both SPI_SEL high
                
 }

//Loading all register values needet for matrix sensor measurement 
void LOAD_CONFIG_5004(void){
    
	SPI_READ_5004_internal_sensors(); //dummy read to assure the PE5004 interrupt is low
    SPI_SEND_5004( 0x02,(WaitCOL>>4),(WaitCOL<<4 | WaitROW >>8),WaitROW);
    SPI_SEND_5004( 0x03,(SINA>>4 | SINF<<2),(PWR>>1 | SINA<<4),(GCO_DACREF | PWR<<7));
    SPI_SEND_5004( 0x04,0x00,(PCO >>8),PCO);

	
}

// processing ADC-Value from PE5004 after H-Signal from Pin INT
void SPI_READ_5004_internal_sensors(void) {
    
	//BYTE SAR_DATA1;
    SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low
	SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //extra delay
	SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //extra delay
	
    SPIM_TX_BUFFER_REG=0x00;  //send adress 0 for SAR-Data register
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    COL_ROW = SPIM_RX_BUFFER_REG; //dummy-read
    
    SPIM_TX_BUFFER_REG=0x00;  //dummy-send for loading next byte
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    COL_ROW = SPIM_RX_BUFFER_REG; // read sensor position
    COL = ( COL_ROW >>4 );
    ROW = ( 0xF & COL_ROW );
    
    SPIM_TX_BUFFER_REG=0x00;  
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    //SAR_DATA[COL][ROW] = (SPIM_RX_BUFFER_REG<<2); // upper 6bits of SAR-DATA
    SAR_DATA = (SPIM_RX_BUFFER_REG<<2); // upper 6bits of SAR-DATA

    SPIM_TX_BUFFER_REG=0x00;
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    //SAR_DATA[COL][ROW] |= (SPIM_RX_BUFFER_REG>>6); // bit 2&3 of SAR-DATA
    SAR_DATA |= (SPIM_RX_BUFFER_REG>>6); // upper 6bits of SAR-DATA

    SPI_SEL_PORT = 0x25;  // pull both SPI_SEL high
	
}

void SPI_READ_5004_external_sensors(void) {
    
	//BYTE SAR_DATA1;
    SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low
	SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //extra delay
	SPI_SEL_PORT = SPI_SEL; //pull SPI_SEL low //extra delay
	
    SPIM_TX_BUFFER_REG=0x00;  //send adress 0 for SAR-Data register
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    COL_ROW = SPIM_RX_BUFFER_REG; //dummy-read
    
    SPIM_TX_BUFFER_REG=0x00;  //dummy-send for loading next byte
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    COL_ROW = SPIM_RX_BUFFER_REG; // read sensor position
    COL = ( COL_ROW >>4 );
    ROW = ( 0xF & COL_ROW );
     	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 
	//SAR_DATA	15	14	13	12	11	10	9	8	7	6	5	4	3	2	  1	   0
	//      	'0'	'0'	------------SAR_DATA------------------ '0'	ECOL  EROW SARMode
	
    SPIM_TX_BUFFER_REG=0x00;  
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    SAR_DATA = SPIM_RX_BUFFER_REG; // middle data byte of reg x00

    SPIM_TX_BUFFER_REG=0x00;
	while( ! (SPIM_CONTROL_REG & SPIM_SPIM_SPI_COMPLETE ) );
    SAR_DATA2 = SPIM_RX_BUFFER_REG; // low data byte of reg x00
				
    SPI_SEL_PORT = 0x25;  // pull both SPI_SEL high
}

    
    
