/********************************************************************
    Project :   PE5004 Evaboard
    Version :   v2.0             
    Software:   PSoC Designer 
    Hardware:   PSoC MiniProg, PE5004 Evaboard
	Device	: 	CY7C64356 
    Date    :   Jan 29 2009
    Author  :   HoJ
    Company :   PE Gmbh               

	desgription: SPI header-file
	
	PORT_1.0	O	strong	SEL0        (BOARD SENSORS)
	PORT_1.1	O	strong	MOSI        (BOARD SENSORS, EXTERNAL SENSORS)
	PORT_1.2	O	strong	SEL1        (EXTERNAL SENSORS)
	PORT_1.3	O	strong	SPI_CLK  (BOARD SENSORS, EXTERNAL SENSORS)
				
	PORT_3.5	I	open drain bypass	MISO        (BOARD SENSORS, EXTERNAL SENSORS)
	PORT_3.0	I	open drain	INT0  (read modify write!)  (BOARD SENSORS)
	PORT_3.1	I	open drain	INT1 (read modify write!)   (EXTERNAL SENSORS)
	
******************************************************************************/

#include <m8c.h> // part specific constants and macros

//Port defines
#define SPI_SEL_PORT PRT1DR //Port 1
#define SPI_SEL0 0x24 //BOARD SENSORS (select low active and MISO High-Z) 
#define SPI_SEL1 0x21 //EXTERNAL SENSORS (select low active and MISO High-Z)

#define INT_PORT PRT3DR //Port3
#define INT0 0x01
#define INT1 0x02

//spi adress defines
//#define SPI_FLAG_5004 0x06000002
//#define START_continous_5004 0x01000003
//#define START_single_5004 0x01000001
//#define STOP_5004 0x00
#define START_5004 0x01

//register value defines for matrix run
#define WaitCOL_M 50;
#define WaitROW_M 10;
#define SINF_M 6;
#define SINA_M 58; //27
#define PWR_M 0x1f; //ref 1f
#define GCO_DACREF_M 0x6f; //ref 4f
#define COLEND_ROWEND_M 0x79;  //79 all
#define COLSTART_ROWSTART_M 0x00;
#define PCO_M 206;


//register value defines for oled run
#define WaitCOL_O 50;
#define WaitROW_O 10;
#define SINF_O 7;
#define SINA_O 26;
#define PWR_O 0x1F;
#define GCO_DACREF_O 0x6F;
#define COLEND_ROWEND_O 0x01;
#define COLSTART_ROWSTART_O 0x00;
#define PCO_O 206;

//register value defines for oled exit run
#define WaitCOL_E 50;
#define WaitROW_E 10;
#define SINF_E 7;
#define SINA_E 21; //26
#define PWR_E 0x1F;
#define GCO_DACREF_E 0x6F;
#define COLEND_ROWEND_E 0x01;
#define COLSTART_ROWSTART_E 0x00;
#define PCO_E 206;

//variable defines
BYTE SPI_SEL = SPI_SEL0;

char buf[4];

BYTE SAR_DATA;
BYTE SAR_DATA2;

//Reg 0 Data
BYTE stepcount=0;
BYTE COL_ROW;
BYTE COL;
BYTE ROW;
//Reg 2 Data
WORD WaitCOL;
WORD WaitROW;
//Reg 3 Data
BYTE SINF;
BYTE SINA;
BYTE PWR;
BYTE GCO_DACREF;
//Reg 4 Data
BYTE COLEND_ROWEND;
BYTE COLSTART_ROWSTART;
WORD PCO;

//-------------------------------------------------
// Prototypes
//-------------------------------------------------

void SPI_SEND_5004(BYTE adress, BYTE send2 ,BYTE send1 ,BYTE send0);

void LOAD_Matrix(void);

void LOAD_Matrix_EXIT(void);

void LOAD_Matrix_USB(void);

void LOAD_OLED(void);

void LOAD_OLED_EXIT(void);

void LOAD_CONFIG_5004(void);
 
void SPI_READ_5004_internal_sensors(void);
    
void SPI_READ_5004_external_sensors(void);

//-------------------------------------------------
// source code
//-------------------------------------------------
#include "spi_functions.c"
