/********************************************************************
    Project :   PE5004 Evaboard
    Version :   v2.0            
    Software:   PSoC Designer 
    Hardware:   PSoC MiniProg, PE5004 Evaboard
	Device	: 	CY7C64356
    Date    :   Jan 29 2009
    Author  :   HoJ
    Company :   PE Gmbh               

	desgription: main source code PE5004 Evakit
		 			"spi_functions.h" 		spi functions
	 				"sensor_processing.h"   sensor functions
	 				"menue_functions.h" 	menue functions
	
******************************************************************************/

#pragma usemac

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "display.h"    // Display Modules
#include <string.h> 
#include <stdlib.h>

//sensor thresholds
BYTE thresh=7; //matrix runs //old 5
BYTE thresh_oled=10; //oles runs // old 4 (v2.0a=6) //(v2.0b=10)
const Oled_exit_tresh = 30; //treshold for oled exit runs
const Matrix_exit_tresh = 20;  //treshold for matrix exit runs

//variable defines
//menue variables
BYTE Loop_var;
BYTE exit_var;
BYTE matrix_exit_var = 0;
BYTE enter_var;
BYTE menue_var = 0;
BYTE count_var = 0;

//cursor variables
WORD pre_r;
WORD pre_c;
WORD rel_r;
WORD rel_c;
BYTE rel_temp_r;
BYTE rel_temp_c;
WORD rel_old_r;
WORD rel_old_c;
WORD summe;
BYTE abs_r;
BYTE abs_c;
BYTE abs_old_r;
BYTE abs_old_c;
BYTE new_touch;
BYTE i_touch;
BYTE i2_touch;
BYTE i3_touch;

//USB variables
BYTE inData[26]; 
BYTE usb_run_var1=0;
BYTE usb_run_var2=0;
BYTE board_external=0;
BYTE sensor_sel_var1;
BYTE sensor_sel_var2;

//includes 
#include "spi_functions.h" //spi functions
#include "sensor_processing.h" //sensor functions
#include "menue_functions.h" //menue functions


//-------------------------------------------------
// main source code
//-------------------------------------------------
void main(void)
{	
	Loop_var = 0x05; 
	SPI_SEL_PORT = 0x25;
	PRT3DR = 0x03; // set open drain pins to Z; disable OLEDs

	//Display INIT
	DISPLAY_INIT_SINGLE_SUPPLY();
	DISPLAY_CLEAR();
	
	//LOGO
	GRAPH_DRAW_LOGO(13, 15, 32, 104);

	//delay for LOGO
	PT0_DATA0 = 0xC0; 
    PT0_DATA1 = 0xFF;
    PT0_CFG = 0x03; 
	while (PT0_CFG & 0x01); // w8
	
	//enable MC SPI
	SPIM_Start(SPIM_SPIM_MODE_0|SPIM_SPIM_MSB_FIRST);  
	
	SPI_SEL = SPI_SEL1; //external sensors
	//set SPI-Flag PE5004 EXTERNAL SENSORS
    SPI_SEND_5004(0x05, 0x00, 0x00, 0x02); 
	
	SPI_SEL = SPI_SEL0; //BOARD SENSORS
	//set SPI-Flag PE5004 BOARD SENSORS
    SPI_SEND_5004(0x05, 0x00, 0x00, 0x02); 
	
	//display calibrating info
	GRAPH_DRAW_STRING_ref(27, 0, "CALIBRATING!");
	GRAPH_DRAW_STRING_ref(27, 56, "Don't touch matrix!");
	
	//init PE5004 for OLED exit calibrating
	LOAD_OLED_EXIT();	
	LOAD_CONFIG_5004();
	CALIBRATE_OLED_EXIT(); //calibrating OLED
	
	//init PE5004 for OLED calibrating
	LOAD_OLED();	
	LOAD_CONFIG_5004();
	CALIBRATE_OLED(); //calibrating OLED
	
	//init PE5004 for MATRIX
	LOAD_Matrix();		
	LOAD_CONFIG_5004();
	CALIBRATE_MATRIX(); //calibrating matrix
	
	//LCD Port init
	PRT2DR = 0b01011111;
	
	//set menue vars
	exit_var = 1;
	sensor_sel_var1=1;
	sensor_sel_var2=1;

	// enable global interrupt
	M8C_EnableGInt; 

	//--USB INIT------------------------------------------------------------------
	USBFS_Start(0, USB_3V_OPERATION); //Start USBFS Operation using device 0  
	                                  //and with 3V operation  
	PT0_DATA0 = 0xff; // 129 cycles
    PT0_DATA1 = 0xff;// 768 cycles
    PT0_CFG = 0x03; // start Timer0 32khz one shot
	while(!USBFS_bGetConfiguration()){ //Wait for Device to enumerate  
	 	if (!(PT0_CFG & 0x01)) break;
	 	}
	//Enumeration is completed load endpoint 1. Do not toggle the first time  	
	USBFS_EnableOutEP(2); // Set up USB device endpoints	
	SAR_RAW[56]=1;  //
	USBFS_LoadInEP(1, SAR_RAW, 57, USB_NO_TOGGLE); //send data to host    
	//--USB INIT end--------------------------------------------------------------	

	//set USB vars
	usb_run_var1=1;
	usb_run_var2=1;



	//start MAIN WHILE ---------------------------------------------------------------------------------
	while(1){  	
		if 	(usb_run_var1==1){
			USBFS_EnableOutEP(2); // Set up USB device endpoint 2 
			usb_run_var1=0;
			}
			
		//Wait for ACK before loading data
		if (USBFS_bGetEPAckState(2)){		
			usb_run_var1=1;
			usb_run_var2=0;			
			USBFS_bReadOutEP(2, inData, 26); //read data from host
			
			if ( inData[10] == 1){ //set external or onboard sensors
				board_external=1;
				SPI_SEL = SPI_SEL0; //BOARD SENSORS
				if (sensor_sel_var1==1){
					sensor_sel_var1=0;
					CLEAR_RAM();
					GRAPH_DRAW_STRING_RAM(40, 20, "USB CONTROL");
					GRAPH_DRAW_STRING_RAM(33, 40, "BOARD SENSORS");
					DISP_SEND_FRAME();
					sensor_sel_var2=1;
					}
				}
			else{
				board_external=2;
				SPI_SEL = SPI_SEL1; //EXTERNAL SENSORS
				if (sensor_sel_var2==1){
					sensor_sel_var2=0;
					CLEAR_RAM();
					GRAPH_DRAW_STRING_RAM(40, 20, "USB CONTROL");
					GRAPH_DRAW_STRING_RAM(25, 40, "EXTERNAL SENSORS");
					DISP_SEND_FRAME();
					sensor_sel_var1=1;
					}
				}

			if (board_external==1){  //onboard sensors selectet? 
				if ( inData[9] == 0xdd){ //OLED sensors selectet when COL_ROW=0xdd
					inData[9] = 0x21; //set COLEND_ROWEND and ignore matrix-field 
					inData[12]= 0x00; //set COLSTART_ROWSTART
					LOAD_Matrix_USB();		
					LOAD_CONFIG_5004();	
					RUN_MATRIX_OLED_USB();  //run OLEDs
					}
				else{ //matrix
					inData[9] = inData[9]+0x01; //ignore oleds
					LOAD_Matrix_USB();	
					LOAD_CONFIG_5004();	
					RUN_MATRIX_OLED_USB();  //run matrix
					}
				}
			else{
				LOAD_Matrix_USB();	
				LOAD_CONFIG_5004();	
				RUN_MATRIX_USB_external_sensors();
				}
			
			//check GUI usb diconnect, inData[11]==1 disconnect
			if (inData[11]==1){ 
				usb_run_var2=1;
				exit_var = 1;
				}
				
			//check if GUI usb connected, inData[11]==2 connect
			if (inData[11]==2){
				USBFS_DisableOutEP(2); //disable USB device endpoint 2 	 
				if (board_external==1){
					while(!USBFS_bGetEPAckState(1));  //Wait for ACK endpoint 1 before loading data 
					USBFS_LoadInEP(1, SAR_RAW, 57, USB_TOGGLE); //send SAR_RAW to USB piont
					}
				else{
					while(!USBFS_bGetEPAckState(1));  //Wait for ACK before loading data 
					USBFS_LoadInEP(1, SAR_RAW, 57, USB_TOGGLE); 
					while(!USBFS_bGetEPAckState(1));  //Wait for ACK before loading data 
					USBFS_LoadInEP(1, SAR_RAW2, 57, USB_TOGGLE); 
				    //SAR_DATA	15	14	13	12	11	10	9	8	7	6	5	4	3	2	  1	    0
			        //      	'0'	'0'	------------SAR_DATA 10bit------------ '0'	ECOL  EROW  SARMode
					pRAW=&SAR_RAW[56];  //   SAR_DATA bits 15..8
					pRAW2=&SAR_RAW2[56]; //   SAR_DATA bits 7..0
					while(!USBFS_bGetEPAckState(1));  //Wait for ACK before loading data 
					USBFS_LoadInEP(1, pRAW, 57, USB_TOGGLE); //send bits 15..8
					while(!USBFS_bGetEPAckState(1));  //Wait for ACK before loading data 
					USBFS_LoadInEP(1, pRAW2, 57, USB_TOGGLE); //send bits 7..0
					}
				}
	 }
//--END USB MAIN---------------------------------------------------------------------
	
	//start menue code
	if (usb_run_var2 ==1){
		sensor_sel_var1=1; 
		sensor_sel_var2=1;
		SPI_SEL = SPI_SEL0; //set to BOARD SENSORS
		M8C_DisableGInt;

		//--Touchpad--------------------------------------------------------------------
		if (exit_var == 1){
			for (count_var=0;count_var<58;count_var++)SAR_FILT[count_var]=1; //reset SAR_FILT
			CLEAR_RAM();
			//display instructions 
			GRAPH_DRAW_STRING_RAM(5, 3, "select:");
			GRAPH_DRAW_STRING_RAM(18, 14, "slider demo");
			GRAPH_DRAW_STRING_RAM(18, 24, "multi touch demo");
			GRAPH_DRAW_STRING_RAM(18, 34, "oled demo");
			GRAPH_DRAW_STRING_RAM(4, 56, "v2.0b");
			GRAPH_DRAW_STRING_RAM(102, 56, "Enter");
			GRAPH_DRAW_STRING_RAM(120, 3, "->");
			GRAPH_DRAW_STRING_RAM(120, 14, "->");
			GRAPH_DRAW_STRING_RAM(120, 24, "->");
			GRAPH_DRAW_STRING_RAM(120, 34, "->");
			GRAPH_DRAW_STRING_RAM(120, 44, "->");
			
			//set cursor display center		
			abs_r = 64; //cursor start values
			abs_c = 4;
			DISPLAY_GRAPH_CURSOR(abs_r, abs_c); 
			
			DISP_SEND_FRAME();
				
			//init variables
			rel_r=0x00; 
			rel_c=0x00;
			rel_old_r=0x00;
			rel_old_c=0x00;	
			new_touch = 1;
			i_touch=0;
			i2_touch=0;
			abs_c = 101;
			abs_r = 94;
			abs_old_c = 101;
			abs_old_r = 94;
			}	
	
		//run sensor matrix read
		LOAD_Matrix();	
		LOAD_CONFIG_5004();	
		//LOAD_CONFIG_5004();	
			
		
		pre_r=0x00;
		pre_c=0x00;
		rel_temp_r=0x00;
		rel_temp_c=0x00;
		summe=0x00;
		//RUN_MATRIX_cursor(); //read matrix 2x
		//RUN_MATRIX_cursor();
		RUN_MATRIX();
		RUN_MATRIX();
		pFILT=&SAR_FILT[0]; //set array start adress
	
		//start touchpad caculation
		//pre calculating matrix "balance point"
		for (runc=1;runc<8;runc++)
			for (runr=1;runr<9;runr++){
				if (*pFILT > 1){
					pre_r += runr**pFILT;
					pre_c += runc**pFILT;
					summe += *pFILT;
					}
				pFILT++;			
			}
			
		//rel_r = 8-(pre_r/summe);
		////rel_c = (pre_c/summe)<<1;
		//rel_c = (pre_c/summe);
		
		// set treshhold 
		//if ((pre_r > 20)|(pre_c > 20))
		if ((summe > 5)&(summe < 250))
			{
			i2_touch=0;
			rel_r = 32-((pre_r<<3)/summe);
			//rel_c = (pre_c/summe)<<1;
			rel_c = ((pre_c<<2)/summe);
			
			
			//rel_r = 8-(pre_r/summe);
			//rel_c = (pre_c/summe)<<1;
	
			if (new_touch == 1){
				rel_old_r = rel_r;
				rel_old_c = rel_c;
		//		abs_old_r = abs_r;
		//		abs_old_c = abs_c;
		//		new_touch = 0;
				}
	
			if (rel_old_r > rel_r){ //if new value lesser than rel_old_r move cursor down
				rel_temp_r = rel_old_r - rel_r;
				//if (rel_temp_r > 0){
					abs_r = abs_old_r - (rel_temp_r);
				//	}
				}
			if (rel_old_r < rel_r){ //if new value larger than rel_old_r move cursor down
				rel_temp_r = rel_r-rel_old_r;
				//if (rel_temp_r > 0){
					abs_r = abs_old_r + (rel_temp_r);
				//	}
				}
			if (rel_old_c > rel_c){ //if new value lesser than rel_old_r move cursor left
				rel_temp_c = rel_old_c - rel_c;
				//if (rel_temp_c > 0){
					abs_c = abs_old_c - (rel_temp_c);
				//	}
				}
			if (rel_old_c < rel_c){ //if new value larger than rel_old_r move cursor right
				rel_temp_c = rel_c-rel_old_c;
				//if (rel_temp_c > 0){
					abs_c = abs_old_c + (rel_temp_c);
				///	}
				}
				
			if (new_touch == 1){
			//	rel_old_r = rel_r;
			//	rel_old_c = rel_c;
				abs_r = abs_old_r ;
				abs_c = abs_old_c ;
				new_touch = 0;
				}	
				

			//stop leaving the display field
			if (abs_r > 150)
				abs_r =150;
			if (abs_r < 40)
				abs_r =40;	
			if (abs_c > 157)
				abs_c =157;
			if (abs_c < 97)
				abs_c =97;
			
			//display selected program witch "->"
			CLEAR_RAM();	
			GRAPH_DRAW_STRING_RAM(5, 3, "select:");
			if ((abs_c > (97+13)) & (abs_c < (97+23))){
				GRAPH_DRAW_STRING_RAM(10, 14, "->slider demo");
				menue_var=1;
				}
			else{
				GRAPH_DRAW_STRING_RAM(18, 14, "slider demo");
				}
			if ((abs_c > (97+23)) & (abs_c < (97+33))){
				GRAPH_DRAW_STRING_RAM(10, 24, "->multi touch demo");
				menue_var=2;
				}
			else{
				GRAPH_DRAW_STRING_RAM(18, 24, "multi touch demo");
				}
			if ((abs_c > (97+33)) & (abs_c < (97+43))){
				GRAPH_DRAW_STRING_RAM(10, 34, "->oled demo");
				menue_var=3;
				}
			else{
				GRAPH_DRAW_STRING_RAM(18, 34, "oled demo");
				}
			GRAPH_DRAW_STRING_RAM(102, 56, "Enter");
			GRAPH_DRAW_STRING_RAM(120, 3, "->");
			GRAPH_DRAW_STRING_RAM(120, 14, "->");
			GRAPH_DRAW_STRING_RAM(120, 24, "->");
			GRAPH_DRAW_STRING_RAM(120, 34, "->");
			GRAPH_DRAW_STRING_RAM(120, 44, "->");
			GRAPH_DRAW_STRING_RAM(4, 56, "v2.0b");
			DISPLAY_GRAPH_CURSOR((abs_r-30), (abs_c-97));
			DISP_SEND_FRAME();
			}
		else{ //find new touch (delay) 
			if(i2_touch==4){ 			//delay   //old 14     
				i2_touch=0;
				//i_touch = 0;
				new_touch = 1;
				}
			i2_touch++;  //increment i2_touch
			if (new_touch == 1){
			//	rel_old_r = rel_r;
			//	rel_old_c = rel_c;
				abs_old_r = abs_r;
				abs_old_c = abs_c;
			//	new_touch = 0;
				}
			}
		
		//reset variables
		enter_var=0;
		exit_var=0;
		matrix_exit_var=0;
	
		//check if touch "ENTER OLED" 
		LOAD_OLED_EXIT();
		//LOAD_CONFIG_5004_OLED_EXIT();	
		//LOAD_CONFIG_5004_OLED_EXIT();	
		LOAD_CONFIG_5004();
		RUN_OLED_EXIT();
		
		//starting selected program
		if ((menue_var==2)&&(enter_var==1)){
			multi_touch_demo();	
			menue_var=0;
			abs_r = 64; //cursor start values
			abs_c = 4;
			DISPLAY_CLEAR();
			CLEAR_RAM();
			DISPLAY_GRAPH_CURSOR(abs_r, abs_c); 
			DISP_SEND_FRAME();
			}
		if ((menue_var==1)&&(enter_var==1)){
			slider_demo();
			menue_var=0;
			abs_r = 64; //cursor start values
			abs_c = 4;
			DISPLAY_CLEAR();
			CLEAR_RAM();
			DISPLAY_GRAPH_CURSOR(abs_r, abs_c); 
			DISP_SEND_FRAME();
			}		
		if ((menue_var==3)&&(enter_var==1)){
			oled_demo();
			menue_var=0;
			abs_r = 64; //cursor start values
			abs_c = 4;
			DISPLAY_CLEAR();
			CLEAR_RAM();
			DISPLAY_GRAPH_CURSOR(abs_r, abs_c); 
			DISP_SEND_FRAME();
			}
		M8C_EnableGInt;	
		}
		//--End menue touchpad----------------------------------------------------------------
	} //end main wihle 
}//end main



